@@ -2,8 +2,8 @@
 
 __author__ = """Benjamin Graedel"""
 __email__ = "benjamin.graedel@unibe.ch"
-__version__ = '0.1.5'
+__version__ = '0.2.1'
 
-from .plotting import NoodlePlot, dataPlots, plotOriginalDetrended, statsPlots
+from ._plotting import NoodlePlot, dataPlots, plotOriginalDetrended, statsPlots
 
 __all__ = ["plotOriginalDetrended", "dataPlots", "statsPlots", "NoodlePlot"]
